@@ -14,7 +14,7 @@ import libcamera
 
 import scicamera.formats as formats
 from scicamera.actions import RequestMachinery
-from scicamera.configuration import CameraConfig, StreamConfig
+from scicamera.configuration import CameraConfig
 from scicamera.controls import Controls
 from scicamera.info import CameraInfo
 from scicamera.lc_helpers import errno_handle, lc_unpack, lc_unpack_controls
@@ -23,11 +23,6 @@ from scicamera.request import CompletedRequest, LoopTask
 from scicamera.sensor_format import SensorFormat
 from scicamera.tuning import TuningContext
 
-STILL = libcamera.StreamRole.StillCapture
-RAW = libcamera.StreamRole.Raw
-VIDEO = libcamera.StreamRole.VideoRecording
-VIEWFINDER = libcamera.StreamRole.Viewfinder
-
 _log = logging.getLogger(__name__)
 
 
@@ -48,6 +43,9 @@ class CameraManager:
         self._lock = threading.Lock()
         self.cms = libcamera.CameraManager.singleton()
 
+    def n_cameras_attached(self):
+        return len(self.cms.cameras)
+
     def get_camera(self, idx: int):
         """Get the (lc) camera with the given index"""
         return self.cms.cameras[idx]
@@ -59,7 +57,8 @@ class CameraManager:
 
     def close_all(self) -> int:
         n_closed = 0
-        for idx in self.cameras.keys():
+        ids_to_close = list(self.cameras.keys())
+        for idx in ids_to_close:
             # FIXME(meawoppl) - this calls back into self.cleanup()
             self.cameras[idx].close()
             n_closed += 1
@@ -86,42 +85,44 @@ class CameraManager:
         sel.register(self.cms.event_fd, selectors.EVENT_READ, self.handle_request)
 
         while self.running:
-            events = sel.select(0.2)
+            events = sel.select(0.05)
             for key, _ in events:
                 callback = key.data
                 callback()
 
         sel.unregister(self.cms.event_fd)
 
-    def handle_request(self, flushid=None):
+    def handle_request(self, flushid: int | None = None) -> int:
         """Handle requests"""
+        n_flushed = 0
         with self._lock:
-            cams = set()
             for req in self.cms.get_ready_requests():
-                if (
-                    req.status == libcamera.Request.Status.Complete
-                    and req.cookie != flushid
-                ):
-                    cams.add(req.cookie)
-                    camera_inst = self.cameras[req.cookie]
-                    cleanup_call = partial(
-                        camera_inst.recycle_request, camera_inst.stop_count, req
-                    )
-                    self.cameras[req.cookie].add_completed_request(
-                        CompletedRequest(
-                            req,
-                            replace(camera_inst.camera_config),
-                            camera_inst.stream_map,
-                            cleanup_call,
-                        )
+                if req.status != libcamera.Request.Status.Complete:
+                    _log.warning("Unexpected request status: %s", req.status)
+                    continue
+                if req.cookie == flushid:
+                    _log.warning("Flushing request.")
+                    n_flushed += 1
+                    continue
+
+                camera_inst = self.cameras[req.cookie]
+                cleanup_call = partial(
+                    camera_inst.recycle_request, camera_inst.stop_count, req
+                )
+                self.cameras[req.cookie].add_completed_request(
+                    CompletedRequest(
+                        req,
+                        replace(camera_inst.camera_config),
+                        camera_inst.stream_map,
+                        cleanup_call,
                     )
+                )
+        return n_flushed
 
 
 class Camera(RequestMachinery):
     """Welcome to the Camera class."""
 
-    _cm = CameraManager()
-
     def __init__(self, camera_num: int = 0, tuning=None):
         """Initialise camera system and open the camera for use.
 
@@ -132,6 +133,7 @@ class Camera(RequestMachinery):
         :raises RuntimeError: Init didn't complete
         """
         super().__init__()
+        self._cm = CameraManager.singleton()
 
         self._cm.add(camera_num, self)
         self.camera_idx = camera_num
@@ -140,72 +142,28 @@ class Camera(RequestMachinery):
         with TuningContext(tuning):
             self._open_camera()
 
-        # Configuration requires various bits of information from the camera
-        # so we build the default configurations here
-        self.preview_configuration = CameraConfig.for_preview(self)
-        self.still_configuration = CameraConfig.for_still(self)
-        self.video_configuration = CameraConfig.for_video(self)
-
-    @property
-    def camera_manager(self):
-        return Camera._cm.cms
-
     def _reset_flags(self) -> None:
         self.camera = None
         self.is_open = False
         self.camera_ctrl_info = {}
         self._preview = None
         self.camera_config = None
-        self.libcamera_config = None
         self.streams = None
         self.stream_map = None
         self.started = False
         self.stop_count = 0
-        self.options = {}
-        self.post_callback = None
         self.camera_properties_ = {}
         self.controls = Controls(self)
         self.sensor_modes_ = None
 
     @property
-    def preview_configuration(self) -> CameraConfig:
-        return self.preview_configuration_
+    def info(self) -> CameraInfo:
+        """Get camera info
 
-    @preview_configuration.setter
-    def preview_configuration(self, value: CameraConfig):
-        assert isinstance(value, CameraConfig)
-        self.preview_configuration_ = value
-
-    @property
-    def still_configuration(self) -> CameraConfig:
-        return self.still_configuration_
-
-    @still_configuration.setter
-    def still_configuration(self, value: CameraConfig):
-        assert isinstance(value, CameraConfig)
-        self.still_configuration_ = value
-
-    @property
-    def video_configuration(self) -> CameraConfig:
-        return self.video_configuration_
-
-    @video_configuration.setter
-    def video_configuration(self, value: CameraConfig):
-        assert isinstance(value, CameraConfig)
-        self.video_configuration_ = value
-
-    @property
-    def asynchronous(self) -> bool:
-        """True if there is threaded operation
-
-        :return: Thread operation state
-        :rtype: bool
+        :return: Camera info
+        :rtype: CameraInfo
         """
-        return (
-            self._preview is not None
-            and getattr(self._preview, "thread", None) is not None
-            and self._preview.thread.is_alive()
-        )
+        return CameraInfo.from_lc_camera(self.camera)
 
     @property
     def camera_properties(self) -> dict:
@@ -223,47 +181,25 @@ class Camera(RequestMachinery):
             for k, v in self.camera_ctrl_info.items()
         }
 
-    def __enter__(self):
-        """Used for allowing use with context manager
-
-        :return: self
-        :rtype: Camera
-        """
-        return self
-
-    def __exit__(self, exc_type, exc_val, exc_traceback):
-        """Used for allowing use with context manager
-
-        :param exc_type: Exception type
-        :type exc_type: Type[BaseException]
-        :param exc_val: Exception
-        :type exc_val: BaseException
-        :param exc_traceback: Traceback
-        :type exc_traceback: TracebackType
-        """
-        self.close()
-
     def __del__(self):
         """Without this libcamera will complain if we shut down without closing the camera."""
         if self.is_open:
             _log.warning(f"__del__ call responsible for cleanup of {self}")
             self.close()
 
-    def requires_camera(self):
+    def _initialize_camera(self) -> None:
+        """Initialize camera
+
+        :raises RuntimeError: Failure to initialize camera
+        """
+        CameraInfo.requires_camera()
+
+        self.camera = self._cm.get_camera(self.camera_idx)
         if self.camera is None:
             message = "Initialization failed."
             _log.error(message)
             raise RuntimeError(message)
 
-    def _initialize_camera(self) -> None:
-        """Initialize camera
-
-        :raises RuntimeError: Failure to initialise camera
-        """
-        CameraInfo.requires_camera()
-        self.camera = self._cm.get_camera(self.camera_idx)
-        self.requires_camera()
-
         self.__identify_camera()
         self.camera_ctrl_info = lc_unpack_controls(self.camera.controls)
         self.camera_properties_ = lc_unpack(self.camera.properties)
@@ -271,14 +207,16 @@ class Camera(RequestMachinery):
         # The next two lines could be placed elsewhere?
         self.sensor_resolution = self.camera_properties_["PixelArraySize"]
         self.sensor_format = str(
-            self.camera.generate_configuration([RAW]).at(0).pixel_format
+            self.camera.generate_configuration([libcamera.StreamRole.Raw])
+            .at(0)
+            .pixel_format
         )
 
         _log.info("Initialization successful.")
 
     def __identify_camera(self):
         # TODO(meawoppl) make this a helper on the camera_manager
-        for idx, address in enumerate(self.camera_manager.cameras):
+        for idx, address in enumerate(self._cm.cms.cameras):
             if address == self.camera:
                 self.camera_idx = idx
                 break
@@ -350,26 +288,19 @@ class Camera(RequestMachinery):
         self._preview = preview
 
     def stop_preview(self) -> None:
-        """Stop preview
-
-        :raises RuntimeError: Unable to stop preview
-        """
+        """Stop preview the preview thread"""
         if not self._preview:
-            raise RuntimeError("No preview specified.")
+            return
 
-        try:
-            self._preview.stop()
-            self._preview = None
-        except Exception:
-            raise RuntimeError("Unable to stop preview.")
+        self._preview.stop()
+        self._preview = None
 
     def close(self) -> None:
         """Close camera
 
         :raises RuntimeError: Closing failed
         """
-        if self._preview:
-            self.stop_preview()
+        self.stop_preview()
         if not self.is_open:
             return
 
@@ -384,64 +315,10 @@ class Camera(RequestMachinery):
         self.camera = None
         self.camera_ctrl_info = None
         self.camera_config = None
-        self.libcamera_config = None
-        self.preview_configuration_ = None
-        self.still_configuration_ = None
-        self.video_configuration_ = None
+        self._preview_configuration = None
         self.allocator = None
         _log.info("Camera closed successfully.")
 
-    # TODO(meawoppl) - Obviated by dataclasses
-    @staticmethod
-    def _update_libcamera_stream_config(
-        libcamera_stream_config, stream_config: StreamConfig, buffer_count: int
-    ) -> None:
-        # Update the libcamera stream config with ours.
-        libcamera_stream_config.size = libcamera.Size(*stream_config.size)
-        libcamera_stream_config.pixel_format = libcamera.PixelFormat(
-            stream_config.format
-        )
-        libcamera_stream_config.buffer_count = buffer_count
-
-    # TODO(meawoppl) - Obviated by dataclasses
-    def _make_libcamera_config(self, camera_config: CameraConfig):
-        # Make a libcamera configuration object from our Python configuration.
-
-        # We will create each stream with the "viewfinder" role just to get the stream
-        # configuration objects, and note the positions our named streams will have in
-        # libcamera's stream list.
-        roles = [VIEWFINDER]
-        main_index, lores_index, raw_index = camera_config.get_stream_indices()
-        if camera_config.lores is not None:
-            roles += [VIEWFINDER]
-        if camera_config.raw is not None:
-            roles += [RAW]
-
-        # Make the libcamera configuration, and then we'll write all our parameters over
-        # the ones it gave us.
-        libcamera_config = self.camera.generate_configuration(roles)
-        libcamera_config.transform = camera_config.transform
-        buffer_count = camera_config.buffer_count
-        self._update_libcamera_stream_config(
-            libcamera_config.at(main_index), camera_config.main, buffer_count
-        )
-        libcamera_config.at(main_index).color_space = camera_config.color_space
-        if camera_config.lores is not None:
-            self._update_libcamera_stream_config(
-                libcamera_config.at(lores_index),
-                camera_config.lores,
-                buffer_count,
-            )
-            libcamera_config.at(lores_index).color_space = camera_config.color_space
-
-        if camera_config.raw is not None:
-            self._update_libcamera_stream_config(
-                libcamera_config.at(raw_index), camera_config.raw, buffer_count
-            )
-            libcamera_config.at(raw_index).color_space = libcamera.ColorSpace.Raw()
-
-        return libcamera_config
-
     def recycle_request(self, stop_count: int, request: libcamera.Request) -> None:
         """Recycle a request.
 
@@ -457,6 +334,10 @@ class Camera(RequestMachinery):
             return
 
         request.reuse()
+
+        # This is where controls on the camera get updated
+        # TODO(meawoppl) - attach a future to he request.cookie
+        # so that we can mature the future when the controls are set
         controls = self.controls.get_libcamera_controls()
         for id, value in controls.items():
             request.set_control(id, value)
@@ -484,55 +365,26 @@ class Camera(RequestMachinery):
                 raise RuntimeError("Could not create request")
 
             for stream in self.streams:
-                if request.add_buffer(stream, self.allocator.buffers(stream)[i]) < 0:
-                    raise RuntimeError("Failed to set request buffer")
+                code = request.add_buffer(stream, self.allocator.buffers(stream)[i])
+                errno_handle(code, "Request.add_buffer()")
             requests.append(request)
+        _log.warning("Made %d requests", len(requests))
         return requests
 
-    def _update_camera_config(
-        self, camera_config: CameraConfig, libcamera_config
-    ) -> None:
-        """Update our camera config from libcamera's.
+    def _config_opts(self, config: dict | CameraConfig) -> CameraConfig:
+        if isinstance(config, CameraConfig):
+            return config
 
-        :param camera_config: Camera configuration
-        :type camera_config: dict
-        :param libcamera_config: libcamera configuration
-        :type libcamera_config: dict
-        """
-        _, lores_index, raw_index = camera_config.get_stream_indices()
-        camera_config.transform = libcamera_config.transform
-        camera_config.color_space = libcamera_config.at(0).color_space
-        camera_config.main = StreamConfig.from_lc_stream_config(libcamera_config.at(0))
-        if lores_index >= 0:
-            camera_config.lores = StreamConfig.from_lc_stream_config(
-                libcamera_config.at(lores_index)
-            )
-        if raw_index >= 0:
-            camera_config.raw = StreamConfig.from_lc_stream_config(
-                libcamera_config.at(raw_index)
-            )
-
-    def _config_opts(self, config: str | dict | CameraConfig) -> CameraConfig:
-        if isinstance(config, str):
-            config_name_to_camera_config = {
-                "preview": self.preview_configuration,
-                "still": self.still_configuration,
-                "video": self.video_configuration,
-            }
-            camera_config = config_name_to_camera_config[config]
-        elif isinstance(config, dict):
+        if isinstance(config, dict):
             _log.warning("Using old-style camera config, please update")
             config = config.copy()
-            config["camera"] = self
-            camera_config = CameraConfig(**config)
-        elif isinstance(config, CameraConfig):
-            # We expect values to have been set for any lores/raw streams.
-            camera_config = config
-        else:
-            raise RuntimeError(f"Don't know how to make a config from {config}")
-        return camera_config
+            return CameraConfig(camera=self, **config)
 
-    def _configure(self, config: str | dict | CameraConfig = "preview") -> None:
+        raise RuntimeError(
+            f"Don't know how to make a config from {config} ({type(config)})"
+        )
+
+    def configure(self, config: dict | CameraConfig) -> None:
         """Configure the camera system with the given configuration.
 
         :param camera_config: Configuration, defaults to the 'preview' configuration
@@ -543,45 +395,21 @@ class Camera(RequestMachinery):
             raise RuntimeError("Camera must be stopped before configuring")
         camera_config = self._config_opts(config)
 
-        if camera_config is None:
-            camera_config = CameraConfig.for_preview(camera=self)
-
         # Mark ourselves as unconfigured.
-        self.libcamera_config = None
         self.camera_config = None
 
         # Check the config and turn it into a libcamera config.
-        libcamera_config = self._make_libcamera_config(camera_config)
-
-        # Check that libcamera is happy with it.
-        status = libcamera_config.validate()
-        self._update_camera_config(camera_config, libcamera_config)
-        _log.debug(f"Requesting configuration: {camera_config}")
-        if status == libcamera.CameraConfiguration.Status.Invalid:
-            raise RuntimeError("Invalid camera configuration: {}".format(camera_config))
-        elif status == libcamera.CameraConfiguration.Status.Adjusted:
-            _log.info("Camera configuration has been adjusted!")
-
-        # Configure libcamera.
-        code = self.camera.configure(libcamera_config)
-        errno_handle(code, "camera.configure()")
-        _log.info("Configuration successful!")
-        _log.debug(f"Final configuration: {camera_config}")
+        camera_config.apply(self.camera)
+        self.stream_map = camera_config.get_stream_map()
 
         # Update the controls and properties list as some of the values may have changed.
         self.camera_ctrl_info = lc_unpack_controls(self.camera.controls)
         self.camera_properties_ = lc_unpack(self.camera.properties)
 
-        indices = camera_config.get_stream_indices()
-        self.stream_map = {}
-        for idx, name in zip(indices, ("main", "lores", "raw")):
-            if idx >= 0:
-                self.stream_map[name] = libcamera_config.at(idx).stream
-        # Record which libcamera stream goes with which of our names.
-        _log.debug(f"Streams: {self.stream_map}")
-
         # Allocate all the frame buffers.
-        self.streams = [stream_config.stream for stream_config in libcamera_config]
+        self.streams = [
+            stream_config.stream for stream_config in camera_config.libcamera_config
+        ]
 
         # TODO(meawoppl) - can be taken off public and used in the 1 function
         # that calls it.
@@ -593,16 +421,11 @@ class Camera(RequestMachinery):
             msg = f"Allocated {len(self.allocator.buffers(stream))} buffers for stream {i}."
             _log.debug(msg)
         # Mark ourselves as configured.
-        self.libcamera_config = libcamera_config
         self.camera_config = camera_config
 
         # Set the controls directly so as to overwrite whatever is there.
         self.controls.set_controls(self.camera_config.controls)
 
-    def configure(self, camera_config="preview") -> None:
-        """Configure the camera system with the given configuration."""
-        self._configure(camera_config)
-
     def camera_configuration(self) -> CameraConfig:
         """Return the camera configuration."""
         return self.camera_config
@@ -628,21 +451,13 @@ class Camera(RequestMachinery):
         self.started = True
         _log.info("Camera started")
 
-    def start(self, config=None) -> None:
+    def start(self) -> None:
         """
         Start the camera system running.
-
-        Camera controls may be sent to the camera before it starts running.
-
-        The following parameters may be supplied:
-
-        config - if not None this is used to configure the camera. This is just a
-            convenience so that you don't have to call configure explicitly.
         """
-        if self.camera_config is None and config is None:
-            config = "preview"
-        if config is not None:
-            self.configure(config)
+        if self.camera_config is None:
+            _log.warning("Camera has not been configured, using preview config")
+            self.configure(CameraConfig.for_preview(self))
         if self.camera_config is None:
             raise RuntimeError("Camera has not been configured")
         # By default we will create an event loop is there isn't one running already.
@@ -664,8 +479,9 @@ class Camera(RequestMachinery):
             # Flush Requests from the event queue.
             # This is needed to prevent old completed Requests from showing
             # up when the camera is started the next time.
-            self._cm.handle_request(self.camera_idx)
+            n_flushed = self._cm.handle_request(self.camera_idx)
             self.started = False
+            _log.warning("Flushed %s requests", n_flushed)
             self._requests = deque()
             _log.info("Camera stopped")
 
@@ -674,10 +490,8 @@ class Camera(RequestMachinery):
         if not self.started:
             _log.debug("Camera was not started")
             return
-        if self.asynchronous:
-            self._dispatch_loop_tasks(LoopTask.without_request(self._stop))[0].result()
-        else:
-            self._stop()
+        self.stop_preview()
+        self._stop()
 
     def set_controls(self, controls) -> None:
         """Set camera controls. These will be delivered with the next request that gets submitted."""
