@@ -50,7 +50,7 @@ class MappedBuffer:
 
 class AbstractCompletedRequest(ABC):
     @abstractmethod
-    def get_config(self, name: str) -> CameraConfig:
+    def get_camera_config(self) -> CameraConfig:
         raise NotImplementedError()
 
     @abstractmethod
@@ -63,10 +63,11 @@ class AbstractCompletedRequest(ABC):
 
     def make_array(self, name: str) -> np.ndarray:
         """Make a 2d numpy array from the named stream's buffer."""
-        config = self.get_config(name)
-        stream_cfg = config.get_config(name)
+        config = self.get_camera_config()
+        stream_cfg = config.get_stream_config(name)
         w, h = stream_cfg.size
         stride = stream_cfg.stride
+        fmt = stream_cfg.format
 
         array = self.get_buffer(name)
 
@@ -74,29 +75,29 @@ class AbstractCompletedRequest(ABC):
         # image stride (which is in bytes) is a whole number of pixels. Even
         # then, if they don't match exactly you will get "padding" down the RHS.
         # Working around this requires another expensive copy of all the data.
-        if config.format in ("BGR888", "RGB888"):
+        if fmt in ("BGR888", "RGB888"):
             if stride != w * 3:
                 array = array.reshape((h, stride))
                 array = np.asarray(array[:, : w * 3], order="C")
             image = array.reshape((h, w, 3))
-        elif config.format in ("XBGR8888", "XRGB8888"):
+        elif fmt in ("XBGR8888", "XRGB8888"):
             if stride != w * 4:
                 array = array.reshape((h, stride))
                 array = np.asarray(array[:, : w * 4], order="C")
             image = array.reshape((h, w, 4))
-        elif config.format in ("YUV420", "YVU420"):
+        elif fmt in ("YUV420", "YVU420"):
             # Returning YUV420 as an image of 50% greater height (the extra bit continaing
             # the U/V data) is useful because OpenCV can convert it to RGB for us quite
             # efficiently. We leave any packing in there, however, as it would be easier
             # to remove that after conversion to RGB (if that's what the caller does).
             image = array.reshape((h * 3 // 2, stride))
-        elif config.format in ("YUYV", "YVYU", "UYVY", "VYUY"):
+        elif fmt in ("YUYV", "YVYU", "UYVY", "VYUY"):
             # These dimensions seem a bit strange, but mean that
             # cv2.cvtColor(image, cv2.COLOR_YUV2BGR_YUYV) will convert directly to RGB.
             image = array.reshape(h, stride // 2, 2)
-        elif config.format == "MJPEG":
+        elif fmt == "MJPEG":
             image = np.array(Image.open(io.BytesIO(array)))
-        elif formats.is_raw(config.format):
+        elif formats.is_raw(fmt):
             image = array.reshape((h, stride))
         else:
             raise RuntimeError("Format " + config.format + " not supported")
@@ -104,7 +105,7 @@ class AbstractCompletedRequest(ABC):
 
     def make_image(self, name: str) -> Image.Image:
         """Make a PIL image from the named stream's buffer."""
-        fmt = self.get_config(name).format
+        fmt = self.get_camera_config().get_stream_config(name).format
         if fmt == "MJPEG":
             buffer = self.get_buffer(name)
             return Image.open(io.BytesIO(buffer))
@@ -166,7 +167,7 @@ class CompletedRequest(AbstractCompletedRequest):
             self.cleanup()
             self.request = None
 
-    def get_config(self, name: str) -> CameraConfig:
+    def get_camera_config(self) -> CameraConfig:
         """Fetch the configuration for the named stream."""
         return self.config
 
